#include "fft_view.h"
#include "OS.h"

fft_view::fft_view(BRect rect, char *title) :
  BView(rect, title, B_FOLLOW_ALL, B_WILL_DRAW |B_PULSE_NEEDED )
{
	rgb_color c;
	int32 ktr;
	bigtime_t seed = system_time();
	srand((int32)seed);
	c.red = rand() % 255;
	c.green = rand() % 255;
	c.blue = rand() % 255;
	SetViewColor(c);

	bar_color.red = (c.red + 128) % 256;
	bar_color.green = (c.green + 128) % 256;
	bar_color.blue = (c.blue + 128) % 256;

	frame_color.red = 0;
	frame_color.green = 0;
	frame_color.blue = 0;
	
	erase_color.red = c.red;
	erase_color.green = c.green;
	erase_color.blue = c.blue;
	
	for (ktr =0; ktr < FFT_BAR_COUNT; ktr++) 
		bar_data[ktr] = bars[ktr] = 0;

}

void fft_view::Pulse()
{
	DrawBars(bars);
}

void fft_view::MouseDown(BPoint where)
{
}

void fft_view::SetBars(uchar *data)
{
	memcpy(bars, data, 256);
}
	
void fft_view::DrawBars(uchar *data)
{
	int32 ktr;
	uint16 old_bar, new_bar;
	Window()->Lock();
	BeginLineArray(FFT_BAR_COUNT);

	for (ktr =0; ktr < FFT_BAR_COUNT; ktr++) {
		if (data) {
			old_bar = bar_data[ktr];
			new_bar = *data++;
			/* Uncomment these for a bit of bar fade. */
			//new_bar = (*data++ * 1.2);
			//new_bar += (old_bar * .8);
			//new_bar *= .5;
			bar_data[ktr] = new_bar;
			if (old_bar != new_bar)
				AddLine(BPoint((ktr*2)+2, (FFT_BAR_RANGE - 2*old_bar+1)),
						BPoint((ktr*2)+2, (FFT_BAR_RANGE - 2*new_bar+1)),
						(old_bar < new_bar ? bar_color:erase_color));
		}
		else
			AddLine(BPoint((ktr*2)+2, FFT_BAR_RANGE), 
					BPoint((ktr*2)+2, FFT_BAR_RANGE - bar_data[ktr]+1), 
					bar_color);
		}
	EndLineArray();
	Window()->Unlock();
}

void fft_view::Draw(BRect where)
{
	BRect b = Bounds();

	DrawBars(NULL);
	BeginLineArray(4);
	AddLine(b.LeftTop(), b.RightTop(), frame_color);
	AddLine(b.RightTop(), b.RightBottom(), frame_color);
	AddLine(b.RightBottom(), b.LeftBottom(), frame_color);
	AddLine(b.LeftBottom(), b.LeftTop(), frame_color);
	EndLineArray();
}


