#include "snd_utils.h"
void stereo2mono(int16 *in, int16 *out, size_t count)
{
	size_t max = count/2;
	int32 ktr;
	for (ktr = 0; ktr < max; ktr++) {
		*out++ = *in;
		in += 2;
	}
}

void mono2stereo(int16 *in, int16 *out, size_t count)
{
	size_t max = count/2;
	int32 ktr;

	for (ktr = 0; ktr < max; ktr++) {
		*out++ = *in;
		*out++ = *in++;
	}
}

void short2float(int16 *in, float *out, size_t count)
{
	for (int ktr = 0; ktr < count; ktr++)
		*out++ = (*in++ / 32768.0);
}


void stereo_short2mono_float(int16 *in, float *out, size_t out_count)
{
	for (int ktr = 0; ktr < out_count; ktr++) {
		*out++ = (*in / 32768.);
		in += 2;
	}
}

void mono_float2stereo_short(float *in, int16 *out, size_t out_count)
{
	int16 hold;
	int32 top = out_count/2;
	for (int ktr = 0; ktr < top; ktr++) {
		*out++ = hold = (*in++ * 32768);
		*out++ = hold;
	}
}
#include "sin.h"
void mono_float2stereo_short_overlap_cos_window(
		float *in1, 
		float *in2, 
		float *in3,
		int16 *out,
		size_t out_count)
{
	float hold;
	int16 *table, *table_last;
	int16 *table_ptr, *table_last_ptr;
	int32 ktr;
	size_t fft_count = out_count/2;
	size_t fft_half_count = fft_count/2;
	
	switch (fft_count) {
		case 4096:
			table = cos_short_half_2048;
			table_last = cos_short_half_2048_last;
			break;
		case 2048:
			table = cos_short_half_1024;
			table_last = cos_short_half_1024_last;
			break;
		case 1024:
			table = cos_short_half_512;
			table_last = cos_short_half_512_last;
			break;
		case 512:
			table = cos_short_half_256;
			table_last = cos_short_half_256_last;
			break;
		default:
			return;
	}

	/* The tables are int16, so the windowing takes care of the
	 * float->int conversion.
	 */
	table_ptr = table;
	table_last_ptr = table_last;

	if (in1) {
		for (ktr = 0; ktr < fft_half_count; ktr++) {
			*out++ = hold = 
					(*in1++ * *table_last_ptr--)  +
					 (*in2++ * *table_ptr++);
			*out++ = hold;
		}
	}
	else {
		for (ktr = 0; ktr < fft_half_count; ktr++) {
			*out++ = hold = (*in2++ * *table_ptr++); 
			*out++ = hold;
		}
	}

	table_ptr = table;
	table_last_ptr = table_last;
	if (in3) {
		for (; ktr < fft_count; ktr++) {
			*out++ = hold = 	
					(*in2++ * *table_last_ptr--)  + 
					 (*in3++ * *table_ptr++);
			*out++ = hold;
		}
	}
	else {
		for (; ktr < fft_count; ktr++) {
			*out++ = hold = (*in2++ * *table_last_ptr--);
			*out++ = hold;
		}
	}	
}

void mono_float2stereo_short_overlap_sin_window(
		float *in1, 
		float *in2, 
		float *in3,
		int16 *out,
		size_t out_count)
{
	float hold;
	int16 *table, *table_last;
	int16 *table_ptr, *table_last_ptr;
	int32 ktr;
	size_t fft_count = out_count/2;
	size_t fft_half_count = fft_count/2;

	switch (fft_count) {
		case 4096:
			table = sin_short_half_2048;
			table_last = sin_short_half_2048_last;
			break;
		case 2048:
			table = sin_short_half_1024;
			table_last = sin_short_half_1024_last;
			break;
		case 1024:
			table = sin_short_half_512;
			table_last = sin_short_half_512_last;
			break;
		case 512:
			table = sin_short_half_256;
			table_last = sin_short_half_256_last;
			break;

		default:
			return;
	}

	/* The tables are int16, so the windowing takes care of the
	 * float->int conversion.
	 */
	table_ptr = table;
	table_last_ptr = table_last;
	if (in1) {
		for (ktr = 0; ktr < fft_half_count; ktr++) {
			*out++ = hold = (max_c(-.99, min_c(.99,(*in1++))) * *table_last_ptr--) +
					 (max_c(-.99,min_c(.99,(*in2++))) * *table_ptr++);
			*out++ = hold;
		}
	}
	else {
		for (ktr = 0; ktr < fft_half_count; ktr++) {
			*out++ = hold = (max_c(-.99,min_c(.99,(*in2++))) * *table_ptr++);	 
			*out++ = hold;
		}
	}

	table_ptr = table;
	table_last_ptr = table_last;
				
	if (in3) {
		for (; ktr < fft_count; ktr++) {
			*out++ = hold = (max_c(-.99,min_c(.99,(*in2++))) * *table_last_ptr--) +
					 (max_c(-.99,min_c(.99,(*in3++))) * *table_ptr++);
			*out++ = hold;
		}
	}
	else {
		for (; ktr < fft_count; ktr++) {
			*out++ = hold = (max_c(-.9,min_c(.9,(*in2++))) * *table_last_ptr--);	 
			*out++ = hold;
		}
	}	
}

#include "fft.h"
void do_fft(float *buf, size_t count, int32 clobber)
{
	int32 ktr;

	fft_real_to_hermitian(buf, count);

	for (ktr = 0; ktr < clobber; ktr++) 
		buf[ktr] = buf[count-1-ktr] = 0.0;
}

void do_inv_fft(float *buf, int32 count)
{
	fftinv_hermitian_to_real(buf, count);
}

#include <math.h>
float magnitude(float real, float imag) 
{
	(float)pow(pow(real,2.0) + pow(imag, 2.0), .5);
}


